/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.featureextraction;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

import com.rapidminer.example.ExampleSet;
import com.rapidminer.imagemining.io.ImageObject;
import com.rapidminer.imagemining.io.ImageObjectList;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.GenerateNewExampleSetMDRule;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.operator.ports.metadata.SimplePrecondition;

/**
 * 
 * Abstract Operator for extracting Features from Images. Given a collection of images via
 * inputport, it extracts the features for each image and returns them as ExampleSet.
 * Use this superclass to implement your own feature extraction operators.
 * 
 * @author Alexander Arimond
 */

public abstract class AbstractFeatureExtractionOperator <F extends Features> extends Operator {

	public static File workingDir = new File(System.getProperty("java.io.tmpdir"), "RMimageMining_" + System.getProperty("user.name")); 
	
	private final InputPort inputPort = getInputPorts().createPort("input", ImageObjectList.class);
	private final OutputPort outputPort = getOutputPorts().createPort("output");
	private final OutputPort original = getOutputPorts().createPort("original");
	
	public AbstractFeatureExtractionOperator(OperatorDescription description) {
		super(description);
		inputPort.addPrecondition(new SimplePrecondition(inputPort, new MetaData(ImageObjectList.class), true));
		GenerateNewExampleSetMDRule rule = new GenerateNewExampleSetMDRule(outputPort){
			public MetaData modifyMetaData(ExampleSetMetaData unmodifiedMD){
				return AbstractFeatureExtractionOperator.this.getResultMetaData(unmodifiedMD);
			}
		};
		getTransformer().addRule(rule);
		getTransformer().addPassThroughRule(inputPort, original);

		workingDir.mkdir();
	}

	public void doWork() throws OperatorException{
		ImageObjectList input = inputPort.getData();
		ExampleSet result;
		try {
			result = extract(input);
		} catch (FeatureExtractionException e) {
			throw new OperatorException("Feature extraction failed for " + e.getImage().getFilename(), e);
		}
		outputPort.deliver(result);
		original.deliver(input);
	}
	
	private ExampleSet extract(ImageObjectList input) throws FeatureExtractionException {
		this.getLogger().entering(this.getClass().toString(), "extract()");
		LinkedList <F> features = new LinkedList <F>();
		for (ImageObject image : input.getList()){
			features.add(extractFeatures(image));
		}
		return convertToExampleSet(features);
	}

	public abstract ExampleSet convertToExampleSet(List<F> features);

	public abstract F extractFeatures(ImageObject image) throws FeatureExtractionException;

	protected abstract MetaData getResultMetaData(ExampleSetMetaData unmodifiedMD);
	
	
}
