/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.featureextraction.mpeg7;

import ij.ImagePlus;

import java.util.List;

import at.lux.imageanalysis.ColorLayoutImpl;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DoubleArrayDataRow;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.imagemining.featureextraction.AbstractFeatureExtractionOperator;
import com.rapidminer.imagemining.featureextraction.FeatureExtractionException;
import com.rapidminer.imagemining.io.ImageObject;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeInt;
import com.rapidminer.parameter.UndefinedParameterError;
import com.rapidminer.tools.Ontology;
import com.rapidminer.tools.att.AttributeSet;

/**
 * Extraction Operator for MPEG-7 Color Layout Descriptors.
 * 
 * @author Alexander Arimond
 */

public class ColorLayoutExtractor extends AbstractFeatureExtractionOperator<ColorLayoutDescriptor>{

	private static final String PARAMETER_YCOEFF = "NumberYCoeff";
	private static final String PARAMETER_CCOEFF = "NumberCCoeff";

	public ColorLayoutExtractor(OperatorDescription description) {
		super(description);
	}

	@Override
	public ExampleSet convertToExampleSet(List<ColorLayoutDescriptor> features) {
		final int dim = getNumberYCoeff() + 2 * getNumberCCoeff();
		AttributeSet set = new AttributeSet();
        final Attribute idAttribute = AttributeFactory.createAttribute("image_ID", Ontology.NOMINAL);
        set.addAttribute(idAttribute);
        final Attribute labelAttribute = AttributeFactory.createAttribute("label", Ontology.NOMINAL);
        set.addAttribute(labelAttribute);
        for (int i = 0; i < getNumberYCoeff(); i++) {
            set.addAttribute(AttributeFactory.createAttribute("yCoeff" + i, Ontology.NUMERICAL));
        }
        for (int i = 0; i< getNumberCCoeff(); i++){
        	set.addAttribute(AttributeFactory.createAttribute("cbCoeff" + i, Ontology.NUMERICAL));
        }
        for (int i = 0; i< getNumberCCoeff(); i++){
        	set.addAttribute(AttributeFactory.createAttribute("crCoeff" + i, Ontology.NUMERICAL));
        }

        // create table
        final MemoryExampleTable table = new MemoryExampleTable(set.getAllAttributes());
        for (ColorLayoutDescriptor descr : features){
        	
        	final double [] dataRow = new double[2 + dim];
        	dataRow [0] = idAttribute.getMapping().mapString(descr.getID());
        	dataRow [1] = labelAttribute.getMapping().mapString(descr.getLabel());
            int j = 2;
        	for (int i = 0; i < getNumberYCoeff(); i++) {
            	dataRow [j] = descr.yCoeff[i];
            	j++;
            }
            for (int i = 0; i< getNumberCCoeff(); i++){
            	dataRow [j] = descr.cbCoeff[i];
            	j++;
            }
            for (int i = 0; i< getNumberCCoeff(); i++){
            	dataRow [j] = descr.crCoeff[i];
            	j++;
            }
        	table.addDataRow(new DoubleArrayDataRow(dataRow));
        }
        final ExampleSet resultSet = table.createExampleSet(set);
        resultSet.getAttributes().setId(idAttribute);
        resultSet.getAttributes().setLabel(labelAttribute);
        return resultSet;
	}

	@Override
	public ColorLayoutDescriptor extractFeatures(ImageObject imageObject)
			throws FeatureExtractionException {
		ImagePlus image = new ImagePlus(imageObject.getFilename());
		ColorLayoutImpl layout = new ColorLayoutImpl(getNumberYCoeff(), getNumberCCoeff(), image.getBufferedImage());
        
		ColorLayoutDescriptor descr = new ColorLayoutDescriptor(layout.getYCoeff(), layout.getCbCoeff(), layout.getCrCoeff(), getNumberYCoeff(), getNumberCCoeff());

		descr.setID(imageObject.getID());
		descr.setLabel(imageObject.getLabel());
		
		return descr;
	}

	private int getNumberYCoeff() {
		try {
			return getParameterAsInt(PARAMETER_YCOEFF);
		} catch (UndefinedParameterError e) {
			// standard is 6
			return 6;
		}
	}

	private int getNumberCCoeff() {
		try {
			return getParameterAsInt(PARAMETER_CCOEFF);
		} catch (UndefinedParameterError e) {
			// standard is 3
			return 3;
		}
	}

	@Override
	protected MetaData getResultMetaData(ExampleSetMetaData unmodifiedMD) {
		// FIXME Auto-generated method stub
		return unmodifiedMD;
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List <ParameterType> types = super.getParameterTypes();
		// FIXME check which min and max numbers are reasonable here!!
		types.add(new ParameterTypeInt(PARAMETER_YCOEFF, "Number of Y coefficients.", 1, 100, 6));
		types.add(new ParameterTypeInt(PARAMETER_CCOEFF, "Number of C coefficients.", 1, 100, 3));
		return types;
	}
	
}
