/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.featureextraction.mpeg7;

import ij.ImagePlus;

import java.util.List;

import at.lux.imageanalysis.DominantColorImplementation;
import at.lux.imageanalysis.DominantColorImplementation.DominantColorValues;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DoubleArrayDataRow;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.imagemining.featureextraction.AbstractFeatureExtractionOperator;
import com.rapidminer.imagemining.featureextraction.FeatureExtractionException;
import com.rapidminer.imagemining.io.ImageObject;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.tools.Ontology;
import com.rapidminer.tools.att.AttributeSet;

/**
 * Extractor for MPEG-7 Dominant Color Descriptors.
 * 
 * @author Alexander Arimond
 */

public class DominantColorExtractor extends AbstractFeatureExtractionOperator<DominantColorDescriptor>{

	public DominantColorExtractor(OperatorDescription description) {
		super(description);
	}

	@Override
	public ExampleSet convertToExampleSet(List<DominantColorDescriptor> features) {
		//final int dim = getNumberYCoeff() + 2 * getNumberCCoeff();
		final AttributeSet set = new AttributeSet();
        final Attribute idAttribute = AttributeFactory.createAttribute("image_ID", Ontology.NOMINAL);
        set.addAttribute(idAttribute);
        final Attribute labelAttribute = AttributeFactory.createAttribute("label", Ontology.NOMINAL);
        set.addAttribute(labelAttribute);
        
        final Attribute spatialCoherencyAttribute = AttributeFactory.createAttribute("spatialCohrency", Ontology.NUMERICAL);
        set.addAttribute(spatialCoherencyAttribute);
        final Attribute percentageAttr = AttributeFactory.createAttribute("percentage", Ontology.NUMERICAL);
        set.addAttribute(percentageAttr);
        // FIXME are these RGB values or something else? change names appropriately.
        final Attribute colorAttr0 = AttributeFactory.createAttribute("colorValue0", Ontology.NUMERICAL);
        set.addAttribute(colorAttr0);
        final Attribute colorAttr1 = AttributeFactory.createAttribute("colorValue1", Ontology.NUMERICAL);
        set.addAttribute(colorAttr1);
        final Attribute colorAttr2 = AttributeFactory.createAttribute("colorValue2", Ontology.NUMERICAL);
        set.addAttribute(colorAttr2);

        // create table
        final MemoryExampleTable table = new MemoryExampleTable(set.getAllAttributes());
        for (DominantColorDescriptor descr : features){
        	
        	final DominantColorValues values = descr.values;

        	final int spatialCoherency = values.getSpatialCoherency();
        	
        	// one row for each color value
        	// FIXME check if this is appropriate representation.. maybe one row for all
        	// Color values would be better.. but number of dominant colors varies for every image.
        	for (int j = 0; j < values.getSize(); j++){
            	final double [] dataRow = new double[7];
            	dataRow [0] = idAttribute.getMapping().mapString(descr.getID());
            	dataRow [1] = labelAttribute.getMapping().mapString(descr.getLabel());
            	
            	dataRow [2] = spatialCoherency;
            	
            	dataRow [3] = values.getPercentage(j);
            	
            	int [] color = values.getColorValue(j); 
            	dataRow [4] = color[0];
            	dataRow [5] = color[1];
            	dataRow [6] = color[2];
            	table.addDataRow(new DoubleArrayDataRow(dataRow));
        	}
        }
        final ExampleSet resultSet = table.createExampleSet(set);
        resultSet.getAttributes().setId(idAttribute);
        resultSet.getAttributes().setLabel(labelAttribute);
        return resultSet;
	}

	@Override
	public DominantColorDescriptor extractFeatures(ImageObject imageObject)
			throws FeatureExtractionException {
		ImagePlus image = new ImagePlus(imageObject.getFilename());
		DominantColorImplementation dominant = new DominantColorImplementation();
		DominantColorDescriptor descr = new DominantColorDescriptor(dominant.extractDescriptor(image.getBufferedImage()));
		
		descr.setID(imageObject.getID());
		descr.setLabel(imageObject.getLabel());
		
		return descr;
	}

	@Override
	protected MetaData getResultMetaData(ExampleSetMetaData unmodifiedMD) {
		// FIXME
		return unmodifiedMD;
	}

}
