/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.featureextraction.mpeg7;

import ij.ImagePlus;

import java.util.List;

import at.lux.imageanalysis.ScalableColorImpl;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DoubleArrayDataRow;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.imagemining.featureextraction.AbstractFeatureExtractionOperator;
import com.rapidminer.imagemining.featureextraction.FeatureExtractionException;
import com.rapidminer.imagemining.io.ImageObject;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeCategory;
import com.rapidminer.parameter.ParameterTypeInt;
import com.rapidminer.parameter.UndefinedParameterError;
import com.rapidminer.tools.Ontology;
import com.rapidminer.tools.att.AttributeSet;

/**
 * Extractor for MPEG-7 Scalable Color descriptors.
 * 
 * @author Alexander Arimond
 */

public class ScalableColorExtractor extends
		AbstractFeatureExtractionOperator<ScalableColorDescriptor> {

	private static final String PARAMETER_COEFF = "number coefficients";
	private static final String PARAMETER_DISCARD_BITS = "bits to discard";

	private static final String [] numCoeffStrings = new String [] {" 16 ", " 32 ", " 64 ", " 128 ", " 256 "};
	private static final int []	   numCoeff		   = new int    [] { 16, 32, 64, 128, 256};
	
	
	public ScalableColorExtractor(OperatorDescription description) {
		super(description);
	}

	@Override
	public ExampleSet convertToExampleSet(List<ScalableColorDescriptor> features) {
		final int dim = getNumCoeff();
		AttributeSet set = new AttributeSet();
        final Attribute idAttribute = AttributeFactory.createAttribute("image_ID", Ontology.NOMINAL);
        set.addAttribute(idAttribute);
        final Attribute labelAttribute = AttributeFactory.createAttribute("label", Ontology.NOMINAL);
        set.addAttribute(labelAttribute);
        for (int i = 0; i < getNumCoeff(); i++) {
            set.addAttribute(AttributeFactory.createAttribute("coeff" + i, Ontology.NUMERICAL));
        }

        // create table
        final MemoryExampleTable table = new MemoryExampleTable(set.getAllAttributes());
        for (ScalableColorDescriptor descr : features){
        	
        	final double [] dataRow = new double[2 + dim];
        	dataRow [0] = idAttribute.getMapping().mapString(descr.getID());
        	dataRow [1] = labelAttribute.getMapping().mapString(descr.getLabel());
            int j = 2;
        	for (int i = 0; i < getNumCoeff(); i++) {
            	dataRow [j] = descr.haarTransformedHistogram[i];
            	j++;
            }
        	table.addDataRow(new DoubleArrayDataRow(dataRow));
        }
        final ExampleSet resultSet = table.createExampleSet(set);
        resultSet.getAttributes().setId(idAttribute);
        resultSet.getAttributes().setLabel(labelAttribute);
        return resultSet;
	}

	@Override
	public ScalableColorDescriptor extractFeatures(ImageObject imageObject)
			throws FeatureExtractionException {
		ImagePlus image = new ImagePlus(imageObject.getFilename());
		ScalableColorImpl scalable = new ScalableColorImpl(image.getBufferedImage(), getNumCoeff(), getBitsDiscarded());
		
		ScalableColorDescriptor descr = new ScalableColorDescriptor(scalable.getHaarTransformedHistogram(), getNumCoeff());
		
		descr.setID(imageObject.getID());
		descr.setLabel(imageObject.getLabel());
		
		return descr;	
	}

	@Override
	protected MetaData getResultMetaData(ExampleSetMetaData unmodifiedMD) {
		// FIXME
		return unmodifiedMD;
	}

	private int getBitsDiscarded() {
		int bitsDiscarded = 0;
		try {
			bitsDiscarded = getParameterAsInt(PARAMETER_DISCARD_BITS);
		} catch (UndefinedParameterError e) {
			// just keep on going with 0
		}
		return bitsDiscarded;
		
	}
	
	private int getNumCoeff(){
		int index = 2;
		try {
			index = getParameterAsInt(PARAMETER_COEFF);
		} catch (UndefinedParameterError e) {
			// cannot occur
			e.printStackTrace();
		}
		return numCoeff[index];
	}
	
	@Override
	public List<ParameterType> getParameterTypes() {
		List <ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeCategory(PARAMETER_COEFF, "Number of coefficients.", numCoeffStrings, 2));
		types.add(new ParameterTypeInt(PARAMETER_DISCARD_BITS, "Number of bits to discard.", 0, 8, 0));
		return types;
	}
	
}
