/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.featureextraction.surf;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DoubleArrayDataRow;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.imagemining.featureextraction.AbstractFeatureExtractionOperator;
import com.rapidminer.imagemining.featureextraction.FeatureExtractionException;
import com.rapidminer.imagemining.io.ImageObject;
import com.rapidminer.imagemining.util.CommandFailedException;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.tools.Ontology;
import com.rapidminer.tools.att.AttributeSet;

/**
 * Extractor for Speeded Up Robust Features (SURF).
 * 
 * More about SURF:
 * Herbert Bay, Andreas Ess, Tinne Tuytelaars, Luc Van Gool
 * "SURF: Speeded Up Robust Features"
 * Computer Vision and Image Understanding (CVIU), Vol. 110, No. 3, pp. 346--359, 2008
 * 
 * TODO this implementation depends on external binaries. Replace by a full JAVA implementation.
 * 
 * @author Alexander Arimond
 */

public class SURF extends AbstractFeatureExtractionOperator<SURFFeatures> {

	public SURF(OperatorDescription description) {
		super(description);
	}

	@Override
	public SURFFeatures extractFeatures(ImageObject image) throws FeatureExtractionException {
		try {
			SURFFeatures features = SURFFeatureExtractor.extractFeatures(new File(image.getFilename()), workingDir);
			features.setID(image.getID());
			features.setLabel(image.getLabel());
			return features;
		} catch (CommandFailedException e) {
			throw new FeatureExtractionException(e, image);
		} catch (IOException e) {
			throw new FeatureExtractionException(e, image);
		} catch (InterruptedException e) {
			throw new FeatureExtractionException(e, image);
		}
		
	}

	@Override
	protected MetaData getResultMetaData(ExampleSetMetaData unmodifiedMD) {
		return new ExampleSetMetaData();
	}

	@Override
	public ExampleSet convertToExampleSet(List<SURFFeatures> results) {
		final int dim = 128;
		AttributeSet set = new AttributeSet();
        final Attribute idAttribute = AttributeFactory.createAttribute("image_ID", Ontology.NOMINAL);
        set.addAttribute(idAttribute);
        final Attribute labelAttribute = AttributeFactory.createAttribute("label", Ontology.NOMINAL);
        set.addAttribute(labelAttribute);
        for (int i = 0; i < dim; i++) {
            set.addAttribute(AttributeFactory.createAttribute("att" + i, Ontology.NUMERICAL));
        }

        // create table
        final MemoryExampleTable table = new MemoryExampleTable(set.getAllAttributes());
        for (SURFFeatures features : results){
        	Iterator <SURFFeatures.SURFFeature> iter = features.iterator();
        	while (iter.hasNext()){
        		SURFFeatures.SURFFeature feat = iter.next();
	        	final double[] dataRow = new double[2 + dim];
	        	
	        	dataRow[0] = idAttribute.getMapping().mapString(features.getID());
	        	dataRow[1] = labelAttribute.getMapping().mapString(features.getLabel());
	        	for (int j = 2; j < dim + 2; j++){
	        		dataRow[j] = feat.values[j -2];
	        	}
	        	table.addDataRow(new DoubleArrayDataRow(dataRow));
        	}
        }
        final ExampleSet resultSet = table.createExampleSet(set);
        resultSet.getAttributes().setId(idAttribute);
        resultSet.getAttributes().setLabel(labelAttribute);
        return resultSet;
	}
	
}
