/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.transformations;

import java.io.File;
import java.util.LinkedList;

import com.rapidminer.imagemining.io.ImageObject;
import com.rapidminer.imagemining.io.ImageObjectList;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.GenerateNewMDRule;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.operator.ports.metadata.SimplePrecondition;

/**
 * Super class for doing image transformations. Given a collection of images, this operator performs
 * some transformation on this image. This will result in a new image with a different ID.
 * 
 * @author Alexander Arimond
 */

public abstract class AbstractImageTransformationOperator extends Operator{
	
	public static File workingDir = new File(System.getProperty("java.io.tmpdir"), "RMimageMining_" + System.getProperty("user.name")); 
	
	private final InputPort inputPort = getInputPorts().createPort("input", ImageObjectList.class);
	private final OutputPort outputPort = getOutputPorts().createPort("output");
	private final OutputPort original = getOutputPorts().createPort("original");
	
	public AbstractImageTransformationOperator(OperatorDescription description) {
		super(description);
		inputPort.addPrecondition(new SimplePrecondition(inputPort, new MetaData(ImageObjectList.class), true));
		getTransformer().addRule(new GenerateNewMDRule(outputPort, ImageObjectList.class));
		getTransformer().addPassThroughRule(inputPort, original);
		
		workingDir.mkdir();
	}

	public void doWork() throws OperatorException{
		ImageObjectList input = inputPort.getData();
		ImageObjectList result = transformImages(input);
		outputPort.deliver(result);
		original.deliver(input);
	}

	private ImageObjectList transformImages(ImageObjectList input) {
		LinkedList <ImageObject> transformedImages = new LinkedList<ImageObject>();
		for (ImageObject image : input.getList()){
			String refinedID = refineID(image.getID());
			
			File resultFile = new File(workingDir, (new File(refinedID)).getName() + "." + getResultType());
			
			transform(new File(image.getFilename()), resultFile);
			transformedImages.add(new ImageObject(resultFile.getAbsolutePath(), refinedID, image.getLabel()));
		}
		return new ImageObjectList(transformedImages);
	}

	/**
	 * The transformed image must get a new ID. This ID might depend on the source image.
	 * Override this method to add your custom ID generation.
	 * 
	 * For example: 
	 * Your source image has ID: image_001
	 * You do a transformation X.
	 * It might be reasonable to have the following ID for the result image: image_001_X
	 * @param ID
	 * @return
	 */
	abstract public String refineID(String ID);
	
	/**
	 * Transforms the image and writes the result into a new image. The Image Mining extension just
	 * delivers the file name of the source image and the result image. The transformation is up
	 * to the transform() method.
	 * 
	 * @param image
	 * @param resultFile
	 */
	abstract public void transform(File image, File resultFile);

	/**
	 * Returns the result type of the transformation operator. Needed for storing the images in
	 * the correct format.
	 * 
	 * For example: Transformation into grayscale would usually result in pgm format
	 * @return
	 */
	
	abstract public String getResultType();
	
}
