/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.transformations;

import ij.ImagePlus;
import ij.io.FileSaver;
import ij.process.ImageConverter;
import ij.process.ImageProcessor;

import java.io.File;

import com.rapidminer.imagemining.io.ImageTypes;

/**
 * Utility class for image transformations. Transformations are realized by using
 * ImageJ library.
 * 
 * @author Alexander Arimond
 */

public class ImageTransformations {

	
	public final static int GRAY_8_BIT = 8;
	public final static int GRAY_16_BIT = 16;
	public final static int GRAY_32_BIT = 32;
	
	public static void transformToGrayScale(File imageFile, File resultFile, String fileType, int bitDepth){
		
		ImagePlus image = new ImagePlus(imageFile.getAbsolutePath());
		
		ImageConverter converter = new ImageConverter(image);
		
		// FIXME other cases?
		switch(bitDepth){
		case (GRAY_8_BIT):
			converter.convertToGray8();
			break;
		case GRAY_16_BIT:
			converter.convertToGray16();
			break;
		case GRAY_32_BIT:
			converter.convertToGray32();
			break;
		}
		save(image, resultFile, fileType);
	}
	
	public static void transformToBinary(File imageFile,  File resultFile, String fileType, int threshold){
		ImagePlus image = new ImagePlus(imageFile.getAbsolutePath());
		ImageConverter converter = new ImageConverter(image);
		
		converter.convertToGray8();
		
		if (threshold == -1){
			image.getProcessor().autoThreshold();
			save(image, resultFile, fileType);
			return;
		} else {
			image.getProcessor().threshold(threshold);
			save(image, resultFile, fileType);
			return;
		}
	}
	
	public static final String DIRECTION_HORIZONTAL = "horizontally";
	public static final String DIRECTION_VERTICAL = "vertically";

	public static void flip(File imageFile, File resultFile, String resultType, String direction) {
		ImagePlus image = new ImagePlus(imageFile.getAbsolutePath());
		
		if (direction.equals(DIRECTION_HORIZONTAL)){
			image.getProcessor().flipHorizontal();
		} else {
			image.getProcessor().flipVertical();
		}
		save(image, resultFile, resultType);
	}

	public static void rotate(File imageFile, File resultFile, String resultType,
			double angle) {
		ImagePlus image = new ImagePlus(imageFile.getAbsolutePath());
		
		image.getProcessor().rotate(angle);

		save(image, resultFile, resultType);
	}
	
	public static int INTERPOLATION_NONE = ImageProcessor.NONE;
	public static int INTERPOLATION_BILINEAR = ImageProcessor.BILINEAR;
	public static int INTERPOLATION_BICUBIC = ImageProcessor.BICUBIC;

	public static void resize(File imageFile, File resultFile, String resultType, int interpolationType, int x, int y){
		ImagePlus image = new ImagePlus(imageFile.getAbsolutePath());
		
		ImageProcessor processor = image.getProcessor();
		processor.setInterpolationMethod(interpolationType);
		
		save(new ImagePlus("", processor.resize(x, y)), resultFile, resultType);
	}
	
	public static void resize(File imageFile, File resultFile, String resultType, int interpolationType, int x){
		ImagePlus image = new ImagePlus(imageFile.getAbsolutePath());
		
		ImageProcessor processor = image.getProcessor();
		processor.setInterpolationMethod(interpolationType);
		
		save(new ImagePlus("", processor.resize(x)), resultFile, resultType);
	}
	
	
	private static void save(ImagePlus image, File resultFile, String fileType) {
		FileSaver saver = new FileSaver(image);
		if (fileType.equals(ImageTypes.PNG))
			saver.saveAsPng(resultFile.getAbsolutePath());
		if (fileType.equals(ImageTypes.JPG))
			saver.saveAsJpeg(resultFile.getAbsolutePath());
		if (fileType.equals(ImageTypes.PGM))
			saver.saveAsPgm(resultFile.getAbsolutePath());
		
		// close image for garbage collection
		image.changes = false;
		image.close();
	}

}
