/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.transformations;

import java.io.File;
import java.util.List;

import com.rapidminer.imagemining.io.ImageTypes;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeCategory;
import com.rapidminer.parameter.ParameterTypeInt;
import com.rapidminer.parameter.UndefinedParameterError;

/**
 * Resizes images to a specified x and y size. Interpolation can be specified as 
 * None, Bilinear or Bicubic. Result format is the source format.
 * 
 * @author Alexander Arimond
 */

public class ResizeOperator extends AbstractImageTransformationOperator {

	public ResizeOperator(OperatorDescription description) {
		super(description);
	}

	private static final String PARAMETER_X = "x";
	private static final String PARAMETER_Y = "y";

	private static final String PARAMETER_INTERPOLATION_TYPE = "interpolation";
	
	private static final String INTERPOLATION_NONE = "None";
	private static final String INTERPOLATION_BILINEAR = "Bilinear";
	private static final String INTERPOLATION_BICUBIC = "Bicubic";
	
	@Override
	public String refineID(String ID) {
		refreshParameters();
		return ID + "_scale" + getX() + "x"+ getY()+ "_" + getInterpolationType();
	}

	private void refreshParameters() {
		try {
			String interpolationTypeString = getParameter(PARAMETER_INTERPOLATION_TYPE);
			if (interpolationTypeString.equals(INTERPOLATION_NONE))
				interpolationType = ImageTransformations.INTERPOLATION_NONE;
			if (interpolationTypeString.equals(INTERPOLATION_BILINEAR))
				interpolationType = ImageTransformations.INTERPOLATION_BILINEAR;
			if (interpolationTypeString.equals(INTERPOLATION_BICUBIC))
				interpolationType = ImageTransformations.INTERPOLATION_BICUBIC;
		} catch (UndefinedParameterError e) {
			interpolationType = ImageTransformations.INTERPOLATION_NONE;
		}
		try {
			x = getParameterAsInt(PARAMETER_X);
		} catch (UndefinedParameterError e) {
			x = 320;
		}
		try {
			y = getParameterAsInt(PARAMETER_Y);
		} catch (UndefinedParameterError e) {
			y = 240;
		}
	}

	private int x;
	private int y;
	private int interpolationType;
	
	private int getX() {
		return x;
	}

	private int getY() {
		return y;
	}

	private int getInterpolationType(){
		return interpolationType;
	}


	@Override
	public void transform(File image, File resultFile) {
		refreshParameters();
		if (getY() == 0){
			ImageTransformations.resize(image, resultFile, getResultType(), getInterpolationType(), getX());
			return;
		} else {
			ImageTransformations.resize(image, resultFile, getResultType(), getInterpolationType(), getX(), getY());
		}
	}

	@Override
	public String getResultType() {
		//FIXME
		return ImageTypes.JPG;
	}
	
	@Override
	public List<ParameterType> getParameterTypes() {
		List <ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeInt(PARAMETER_X, "resize x", 1, 10000, 320));
		types.add(new ParameterTypeInt(PARAMETER_Y, "resize y (optional)", 0, 10000, 0, true));
		types.add(new ParameterTypeCategory(PARAMETER_INTERPOLATION_TYPE, "Interpolation method: None, Bilinear, Bicubic", new String[] {INTERPOLATION_NONE, INTERPOLATION_BILINEAR, INTERPOLATION_BICUBIC}, 0));
		return types;
	}

}
